package com.baidu.util.crypt;


import java.lang.ref.WeakReference;

import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;




import com.baidu.utility.LogHelper;



/**
 * @author lg
 *
 */
public class AudioEncrypt {

    private final static String tag = "AudioEncrypt";
    private static final LogHelper logHelper = new LogHelper(tag);
    
    private int mNativeContext; // accessed by native methods

    AudioEncryptListener mEncryptListener = null;
    private EventHandler mEventHandler;
    
    static {
        System.loadLibrary("basecore");
        System.loadLibrary("audiocrypt");
        native_init();
    }

    public AudioEncrypt(Looper eventlooper) {
        if (eventlooper != null) {
            logHelper.infoShow("AudioEncrypt eventlooper out");
            mEventHandler = new EventHandler(this, eventlooper);
        } else {
            logHelper.infoShow("AudioPlayer eventlooper null");
            Looper looper;
            if ((looper = Looper.myLooper()) != null) {
            	logHelper.infoShow("AudioPlayer eventlooper my looper");
                mEventHandler = new EventHandler(this, looper);
            } else if ((looper = Looper.getMainLooper()) != null) {
            	logHelper.infoShow("AudioPlayer eventlooper main looper");
                mEventHandler = new EventHandler(this, looper);
            } else {
                mEventHandler = null;
            }
        }

        /*
         * Native setup requires a weak reference to our object. It's easier to
         * create it here than in C++.
         */
        native_setup(new WeakReference<AudioEncrypt>(this));
    }

    public AudioEncrypt() {
        this(null);
    }
    
    
    /**
     * check whether the audio file filePath is encrypted
     * @param filePath
     * @return
     */
    public static boolean isFileEncrypted(String filePath) {
    	logHelper.debugShow("java isFileEncrypted " + filePath);
    	return native_isFileEncrypted(filePath);
    }

    // 音频资源类型
    public final static int ACEncryptSourceTypeNotEncrypt = 0;	// 未加密
    public final static int ACEncryptSourceTypeTencent = 1; 	// 腾讯资源

    
    /**
     * @param filePath
     * @return ACEncryptSourceTypeTencent or ACEncryptSourceTypeNotEncrypt(未加密)
     */
    public static int getFileEncryptedType(String filePath) {
    	logHelper.debugShow("java getFileEncryptedType " + filePath);
    	return native_getFileEncryptedType(filePath);
    }
    
    /**
     * @param filePath
     * @return ACEncryptSourceTypeTencent or ACEncryptSourceTypeNotEncrypt(未加密)
     */
    public static EncryptFileInfo getFileEncryptedInfos(String filePath) {
    	logHelper.debugShow("java getFileEncryptedInfos " + filePath);
    	return native_getFileEncryptedInfos(filePath);
    }

    /**
     * set temp directory for internal use. 
     * this directory should be created by app, this method will create tmpDir if tmpDir not exist.
     * @param tmpDir  
     * @throws IllegalStateException
     */
    public static void setTmpFolder(String tmpDir) throws IllegalStateException {
    	native_setTmpFolder(tmpDir);
    }
    
    public void setEncryptListener(AudioEncryptListener listener) {
    	logHelper.debugShow("java setEncryptListener");
        this.mEncryptListener = listener;
    }
    
    public void releaseEncrypt() {
        logHelper.debugShow("java releaseEncrypt");
        native_release();
    }

    
    /**
     * encrypt local audio file, srcFilePath and destFilePath can be same.
     * original file will be replaced by encrypted file if success,
     * original file will remain if failed
     * @param sourceType	ACEncryptSourceTypeTencent
     * @param srcFilePath	original audio file path
     * @param destFilePath  encrypt audio file path, directory must exist 
     * @param songid   歌曲songid 无songid传入0
     */
    public void encryptAudio(int sourceType, String srcFilePath, String destFilePath, long songid) throws IllegalStateException, IllegalArgumentException {
        logHelper.debugShow("java encryptAudio sourceType:" + sourceType 
                + " srcFilePath: " + srcFilePath 
                + " destFilePath: " + destFilePath
                + " songid: " + songid);
        native_encrypt(sourceType, srcFilePath, destFilePath, songid);
    }

    /**
     * encrypt local audio file, srcFilePath and destFilePath can be same.
     * original file will be replaced by encrypted file if success,
     * original file will remain if failed
     * @param sourceType	ACEncryptSourceTypeTencent
     * @param srcFilePath	original audio file path
     * @param destFilePath  encrypt audio file path, directory must exist 
     */
    public void encryptAudio(int sourceType, String srcFilePath, String destFilePath) throws IllegalStateException, IllegalArgumentException {
        logHelper.debugShow("java encryptAudio sourceType:" + sourceType 
                + " srcFilePath: " + srcFilePath 
                + " destFilePath: " + destFilePath);
        native_encrypt(sourceType, srcFilePath, destFilePath, 0);
    }

    
    /**
     * intterupt encrypt
     */
    public void interrupt() throws IllegalStateException {
    	logHelper.debugShow("java interrupt");
    	native_interrupt();
    }
    /*
     * Do not change these values without updating their counterparts in
     * ac_define.h!
     */
    private static final int NOTIFY_CRYPT_PROGRESS = 700;
    private static final int NOTIFY_CRYPT_RESULT = 701;
    

    private class EventHandler extends Handler {
        private AudioEncrypt mAudioPlayer;

        public EventHandler(AudioEncrypt ap, Looper looper) {
            super(looper);
            mAudioPlayer = ap;
        }

        @Override
        public void handleMessage(Message msg) {
            if (mAudioPlayer.mNativeContext == 0) {
                logHelper
                        .warnShow("mediaplayer went away with unhandled events");
                return;
            }

            logHelper.infoShow("msg = " + msg.what  + " msg.arg1 " + msg.arg1 + " begin");
            switch (msg.what) {
            case NOTIFY_CRYPT_PROGRESS:
            {
                int progress = msg.arg1;
                if (mEncryptListener != null)
                    mEncryptListener.onEncryptProgress(progress);
            }
                 break;
            case NOTIFY_CRYPT_RESULT:
            {
                int code = msg.arg1;
                if (mEncryptListener != null)
                    mEncryptListener.onEncryptResult(code);
            }
                 break;
           default:
                logHelper.errorShow("Unknown message type " + msg.what);
                break;
            }

            logHelper.infoShow("msg = " + msg.what + " end");
        }
    }

    private void SendMessage(int what, int arg1, int arg2, Object obj) {
        if (mEventHandler != null) {
            Message m = mEventHandler.obtainMessage();
            m.what = what;
            m.arg1 = arg1;
            m.arg2 = arg2;
            m.obj = obj;
            mEventHandler.sendMessage(m);
        }
    }

    /**
     * Called from native code when an interesting event happens. This method
     * just uses the EventHandler system to post the event back to the main app
     * thread. We use a weak reference to the original AudioPlayer object so
     * that the native code is safe from the object disappearing from underneath
     * it. (This is the cookie passed to native_setup().)
     */
    private static void postEventFromNative(Object mediaplayer_ref, int what,
            int arg1, int arg2, Object obj) {

//        logHelper.infoShow("postEventFromNative");
        AudioEncrypt ap = (AudioEncrypt) ((WeakReference<?>) mediaplayer_ref)
                .get();
        if (ap == null) {
            logHelper.infoShow("postEventFromNative ap null");
            return;
        }

        if (ap.mEventHandler != null) {
//            logHelper.infoShow("postEventFromNative message");
            Message m = ap.mEventHandler.obtainMessage(what, arg1, arg2, obj);
            ap.mEventHandler.sendMessage(m);
        }
    }

    private static native final void native_init();
    
    private static native boolean native_isFileEncrypted(String filePath);
    
    private static native int native_getFileEncryptedType(String filePath);
    
    private static native EncryptFileInfo native_getFileEncryptedInfos(String filePath);
    
    private native void native_interrupt() throws IllegalStateException;
    
    private native final void native_setup(Object mediaplayer_this);
    
    private native void native_encrypt(int sourceType, String srcFilePath, String destFilePath, long songid) throws IllegalStateException, IllegalArgumentException;
        
    private native void native_release(); 
    
    private static native void native_setTmpFolder(String tmpDir) throws IllegalStateException;
}
