package com.baidu.util.learncore;

import android.media.AudioFormat;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;

import java.io.FileDescriptor;
import java.io.IOException;

import com.baidu.musicPlayer.AudioTrackPlayer;
import com.baidu.musicRecorder.RecordError;
import com.baidu.musicRecorder.RecorderManager;
import com.baidu.utility.LogHelper;
import com.utility.publicInterface.RecordListener;

public class LearnPlayer implements RecordListener {

    private static final String tag = "LearnPlayer";
    private static final LogHelper logHelper = new LogHelper(tag);

    private static final int BASE_INT = 0x01 << 5;
    public static final int INFO_HIGH_PITCH = BASE_INT + 1;
    public static final int INFO_GOOD_SINGER = BASE_INT + 2;
    public static final int DEFAULT_RECORD_SAMPLERATE = 44100;

    private int mNativeContext = 0;
    private AudioTrackPlayer mAudioTrackPlayer = null;
    private int mLearnMode = -1;
    private int mCalcLatencyIndex = 0;
    private int mLatencyMS = 0;
    private static final int LATENCY_COUNT = 5;
    private static final int LATENCY_ADJ = 90;
    private static final int PLAY_FIX_LATENCY = 60;

    private EventHandler mEventHandler;

    private boolean mIsRecycle = false;
    private int mDurationMillisec = 0;

    private int mFlags = 0;

    // private static final boolean LOGGABLE = false; // release version
    // // private static final boolean LOGGABLE = true; // debug version

    static {
        System.loadLibrary("coreffmpeg");
        System.loadLibrary("ijksoundtouch");
        System.loadLibrary("basecore");
        System.loadLibrary("songlearn");
        native_init();
    }

    public LearnPlayer() {
        logHelper.infoShow("init");
        native_setup(this); // set context
        onInit();
    }

    private void clearPlayFlags() {
        mFlags = 0;
    }

    private void setPlayerFlagsByIndex(int nIndex, boolean bFlag) {
        mFlags |= (bFlag ? 1 : 0) << nIndex;
    }

    private void setPlayerFlags() {
        native_setPlayerFlags(mFlags);
    }

    public void cancelEchoProcess(boolean bCancel) {
        native_cancelEchoProcess(bCancel);
    }

    /*
     * 设置日志
     */
    public static void enableGlobalLog(boolean loggable) {
        logHelper.enableLog(loggable);
        native_enableGlobalLog(loggable);
    }

    /**
     * 设置音频处理临时路径.
     * 
     * @param strPath 指定音频处理的临时路径
     */
    public static boolean setTempPath(String strPath) {
        return native_setTempPath(strPath);
    }

    public boolean setLyricPath(String strPath) {
        return native_setLyricPath(strPath);
    }

    /**
     * 设置伴奏源文件,只需要设置一次，reset后不需要重新设置 Sets the data source (FileDescriptor) to use. It is the caller's responsibility to
     * close the file descriptor. It is safe to do so as soon as this call returns.
     * 
     * @param fd the FileDescriptor for the file you want to play
     * @throws IllegalStateException if it is called in an invalid state
     */
    public void setDataSource(FileDescriptor fd) throws IOException, IllegalArgumentException, IllegalStateException {
        // intentionally less than LONG_MAX
        native_setDataSource(fd, 0, 0x7fffffffL);
    }

    public void setDataSource(String strFilePath) throws IllegalArgumentException, IllegalStateException, IOException {
        native_setDataSource(strFilePath);
    }

    public void setMixRealTimeEnable(boolean bRealTime) {
        RecorderManager.sharedPreferenceRecorderManager().setSingWhenRecording(bRealTime);
    }

    public interface OnHowToSingListener {
        public void onGoodVoiceSing(int nNotifyType);
    }

    private OnHowToSingListener mOnHowToSingListener = null;

    public void setOnHowToSingListener(OnHowToSingListener listener) {
        mOnHowToSingListener = listener;
    }

    /**
     * 监控录音音量.
     * 
     * @see com.baidu.util.learncore.LearnPlayer.OnRecordVolumeChangeListener
     */
    public static final int RECORDVOLUME_HIGH = 0;
    public static final int RECORDVOLUME_LOW = 1;

    /**
     * 监控录制声音音量.
     */
    public interface OnRecordVolumeChangeListener {
        /**
         * 当录音音量变足够大或变足够小时，上层通知切换原唱(录音音量小)伴奏(录音音量大)
         * 
         * @param ap the LearnPlayer the error pertains to
         * @param what the type of error that has occurred:
         *            <ul>
         *            <li>{@link #RECORDVOLUME_HIGH}
         *            <li>{@link #RECORDVOLUME_LOW}
         *            </ul>
         * @return True if the method handled the error, false if it didn't. Returning false, or not having an
         *         OnErrorListener at all, will cause the OnCompletionListener to be called.
         */
        void onRecordVolumeChange(LearnPlayer ap, int what);
    }

    /**
     * Register a callback to be invoked when an error has happened during an asynchronous operation.
     * 
     * @param listener the callback that will be run
     */
    public void setOnRecordVolumeChangeListener(OnRecordVolumeChangeListener listener) {
        mOnRecordVolumeChangeListener = listener;
    }

    private OnRecordVolumeChangeListener mOnRecordVolumeChangeListener;

    /**
     * Interface definition of a callback to be invoked indicating processing status of a media resource being processed
     * when saving
     */
    public interface OnSavingUpdateListener {
        /**
         * Called to update status in saving mode.
         * 
         * @param ap the LearnPlayer the update pertains to
         * @param percent the percentage (0-100) of the saving process that has been filled thus far
         */
        void onSavingUpdate(LearnPlayer ap, int percent);
    }

    /**
     * Register a callback to be invoked when the status of a saving percent has changed.
     * 
     * @param listener the callback that will be run.
     */
    public void setOnSavingUpdateListener(OnSavingUpdateListener listener) {
        mOnSavingUpdateListener = listener;
    }

    private OnSavingUpdateListener mOnSavingUpdateListener;

    /**
     * Interface definition of a callback to be invoked indicating the completion of a seek operation.
     */
    public interface OnSeekCompleteListener {
        /**
         * Called to indicate the completion of a seek operation.
         * 
         * @param ap the LearnPlayer that issued the seek operation
         * @param suc the result of seek operation: true success, false fail;
         */
        public void onSeekComplete(LearnPlayer ap, boolean suc);
    }

    /**
     * Register a callback to be invoked when a seek operation has been completed.
     * 
     * @param listener the callback that will be run
     */
    public void setOnSeekCompleteListener(OnSeekCompleteListener listener) {
        mOnSeekCompleteListener = listener;
    }

    private OnSeekCompleteListener mOnSeekCompleteListener;

    /**
     * Unspecified media player error.
     * 
     * @see com.baidu.util.learncore.LearnPlayer.OnErrorListener
     */
    public static final int MEDIA_ERROR_UNKNOWN = 1;
    public static final int MEDIA_ERROR_AUDIO_PARSE = 4; // 音频格式解析错误
    public static final int MEDIA_ERROR_AUDIO_WIRTE = 5; // 音频文件写入错误
    public static final int MEDIA_ERROR_AUDIO_CODEC = 6; // 音频解码错误
    public static final int MEDIA_ERROR_AUDIO_NOIMP = 7; // 该功能暂未实现
    public static final int MEDIA_ERROR_AUDIO_UNKNOWN = 8; // 底层未知错误
    public static final int MEDIA_ERROR_AUDIO_SCORE = 11; // 评分模块错误
    public static final int MEDIA_ERROR_AUDIO_INVALID_RES = 12; // 伴奏资源格式错误
    public static final int MEDIA_ERROR_AUDIO_INVALID_LYC = 13; // 歌词资源格式错误
    // 录音错误
    public static final int MEDIA_ERROR_RECORD_FMT = 100; // 录音格式错误
    public static final int MEDIA_ERROR_RECORD_HW = 101; // 录音设备被占用
    public static final int MEDIA_ERROR_RECORD_UNINIT = 102; // android系统返回录音未能初始化，比如录音权限被禁止会有该错误

    public interface OnErrorListener {
        /**
         * Called to indicate an error.
         * 
         * @param ap the LearnPlayer the error pertains to
         * @param what the type of error that has occurred:
         *            <ul>
         *            <li>{@link #MEDIA_ERROR_UNKNOWN}
         *            <li>{@link #MEDIA_ERROR_RECORD_FMT}
         *            <li>{@link #MEDIA_ERROR_RECORD_HW}
         *            <li>{@link #MEDIA_ERROR_AUDIO_PARSE}
         *            <li>{@link #MEDIA_ERROR_AUDIO_WIRTE}
         *            <li>{@link #MEDIA_ERROR_AUDIO_CODEC}
         *            <li>{@link #MEDIA_ERROR_AUDIO_NOIMP}
         *            <li>{@link #MEDIA_ERROR_AUDIO_UNKNOWN}
         *            <li>{@link #MEDIA_ERROR_AUDIO_SCORE}
         *            <li>{@link #MEDIA_ERROR_AUDIO_INVALID_RES}
         *            <li>{@link #MEDIA_ERROR_AUDIO_INVALID_LYC}
         *            <li>{@link #MEDIA_ERROR_RECORD_UNINIT}
         *            </ul>
         * @param extra an extra code, specific to the error. Typically implementation dependant.
         * @return True if the method handled the error, false if it didn't. Returning false, or not having an
         *         OnErrorListener at all, will cause the OnCompletionListener to be called.
         */
        boolean onError(LearnPlayer ap, int what, int extra);
    }

    /**
     * Register a callback to be invoked when an error has happened during an asynchronous operation.
     * 
     * @param listener the callback that will be run
     */
    public void setOnErrorListener(OnErrorListener listener) {
        mOnErrorListener = listener;
    }

    private OnErrorListener mOnErrorListener;

    /**
     * Unspecified media player info.
     * 
     * @see com.baidu.util.learncore.LearnPlayer.OnInfoListener
     */
    public static final int MEDIA_INFO_UNKNOWN = 1;

    /**
     * A new set of metadata is available.
     * 
     * @see com.baidu.util.learncore.LearnPlayer.OnInfoListener
     */
    public static final int MEDIA_INFO_RENDER_START = 900;

    public static final int MEDIA_INFO_RENDER_END = 901;

    /**
     * Interface definition of a callback to be invoked to communicate some info and/or warning about the media or its
     * playback.
     */
    public interface OnInfoListener {
        /**
         * Called to indicate an info or a warning.
         * 
         * @param ap the LearnPlayer the info pertains to.
         * @param what the type of info or warning.
         *            <ul>
         *            <li>{@link #MEDIA_INFO_UNKNOWN}
         *            <li>{@link #MEDIA_INFO_RENDER_START}
         *            <li>{@link #MEDIA_INFO_RENDER_END}
         *            </ul>
         * @param extra an extra code, specific to the info. Typically implementation dependant.
         * @return True if the method handled the info, false if it didn't. Returning false, or not having an
         *         OnErrorListener at all, will cause the info to be discarded.
         */
        boolean onInfo(LearnPlayer ap, int what, int extra);
    }

    /**
     * Register a callback to be invoked when an info/warning is available.
     * 
     * @param listener the callback that will be run
     */
    public void setOnInfoListener(OnInfoListener listener) {
        mOnInfoListener = listener;
    }

    private OnInfoListener mOnInfoListener;

    /**
     * Interface definition for a callback to be invoked when complete.
     */
    public interface OnCompletionListener {
        /**
         * Called when the end of a media source is reached.
         * 
         * @param ap the AudioPlayer that reached the end of the file
         */
        void onCompletion(LearnPlayer ap);
    }

    /**
     * Register a callback to be invoked when the end of a mode has been reached.
     * 
     * @param listener the callback that will be run
     */
    public void setOnCompletionListener(OnCompletionListener listener) {
        mOnCompletionListener = listener;
    }

    private OnCompletionListener mOnCompletionListener;

    private void sleepTime(long nMills) {
        try {
            Thread.sleep(nMills);
        } catch (InterruptedException e) {
            // TODO 自动生成的 catch 块
            e.printStackTrace();
        }
    }

    /**
     * 学歌中的几个模式，不同模式对应的方法含义可能不同.
     * 
     * @see com.baidu.util.learncore.LearnPlayer.prepare
     */
    public static final int LEARNPLAYER_MODE_RECORD = 0; // 录音学歌

    public static final int LEARNPLAYER_MODE_LISTEN = 1; // 试听

    public static final int LEARNPLAYER_MODE_SAVEMIX = 2; // 保存混音

    public static final int LEARNPLAYER_MODE_SAVEVOICE = 3; // 保存人声

    /**
     * 准备 录音模式准备录音+播放，试听模式准备混音，保存模式准备保存 同步接口，除设置路径外的其他设置需要在prepare后调用
     * 
     * @param mode 指定学歌流程中的几个模式
     * @throws IllegalStateException if it is called in an invalid state
     */
    public void prepare(int mode) throws IOException, IllegalStateException {
        mLearnMode = mode;
        native_prepare(mode);
        mDurationMillisec = getDuration();
        if (mLearnMode == LEARNPLAYER_MODE_RECORD) {
            sleepTime(100);// lg temp
        }
        if (mLearnMode == LEARNPLAYER_MODE_RECORD || mLearnMode == LEARNPLAYER_MODE_LISTEN) {
            mAudioTrackPlayer = new AudioTrackPlayer(this);
            mAudioTrackPlayer.setSampleRate(native_getRenderSampleRate());
            mAudioTrackPlayer.setBitsPerSample(native_getRenderBitsPerSample());
            mAudioTrackPlayer.setChannels(native_getRenderChannels());
            mAudioTrackPlayer.prepare();
        }

        if (mLearnMode == LEARNPLAYER_MODE_RECORD) {
            RecorderManager.sharedPreferenceRecorderManager().setRecordListener(this);

            RecorderManager.sharedPreferenceRecorderManager()
            // .setRecordSettings(native_getRenderSampleRate(),
            // //native_getRenderSampleRate() all record samplerate set to
            // 44100, all devices support 44100, latency is smaller than 22050
                    .setRecordSettings(DEFAULT_RECORD_SAMPLERATE, // native_getRenderSampleRate()
                                                                  // all record
                            // samplerate set to 44100, all devices
                            // support 44100, latency is smaller than
                            // 22050
                            AudioFormat.CHANNEL_IN_MONO, AudioFormat.ENCODING_PCM_16BIT);
            // RecorderManager.sharedPreferenceRecorderManager().setSingWhenRecording(true);
            if (!RecorderManager.sharedPreferenceRecorderManager().prepareRecord()) {
                throw new IllegalStateException("recorder not prepared !");
            }
            RecorderManager.sharedPreferenceRecorderManager().create();

            mLatencyMS = 0;
            mCalcLatencyIndex = 0;
        }
    }

    /**
     * 准备,仅录音模式使用
     * 
     * @param bKeepRecordData 是否保持现有录音
     * @throws IllegalStateException if it is called in an invalid state
     */
    public void prepare(int mode, boolean bKeepRecordData) throws IOException, IllegalStateException {
        native_keepRecordData(bKeepRecordData);
        prepare(mode);
    }

    /**
     * 开始 录音模式开始录音+播放，可以从暂停中恢复； 试听模式开始播放混音，可以从暂停中恢复； 保存混音模式开始保存，暂不支持暂停恢复； 保存人声模式开始保存，暂不支持暂停恢复；
     * 
     * @throws IllegalStateException if it is called in an invalid state
     */
    public void start() throws IllegalStateException {
        native_start();
        if (mAudioTrackPlayer != null) {
            mAudioTrackPlayer.start();
        }
        if (mLearnMode == LEARNPLAYER_MODE_RECORD) {
            RecorderManager.sharedPreferenceRecorderManager().startRecord();
        }
    }

    /**
     * 暂停 录音模式暂停录音+播放； 试听模式暂停播放混音； 保存模式暂不支持；
     * 
     * @throws IllegalStateException if the internal player engine has not been initialized.
     */
    public void pause() throws IllegalStateException {
        if (mAudioTrackPlayer != null) {
            mAudioTrackPlayer.pause();
        }

        if (mLearnMode == LEARNPLAYER_MODE_RECORD) {
            RecorderManager.sharedPreferenceRecorderManager().pauseRecord();
        }
        native_pause();
    }

    /**
     * 定位 录音模式定位到开始录音的时间点msec，播放时间点为msec+msecOffset； 试听模式定位到指定位置播放混音，播放时间点msec，msecOffset忽略；
     * 保存模式不支持定位。异步OnSeekCompleteListener
     * 
     * @param msec 距离起始位置的偏移时间，单位毫秒
     * @param msecOffset 距离msec位置的偏移时间，单位毫秒，在msec之前为负值，之后为正值
     * @throws IllegalStateException if the internal player engine has not been initialized
     */
    public void seekTo(int msec, int msecOffset) throws IllegalStateException {
        logHelper.infoShow("learnplayer seekTo start msec:" + msec + "msecOffset:" + msecOffset);

        int nLatencyAdj = 0;
        if (mAudioTrackPlayer != null) {

            if (mLearnMode == LEARNPLAYER_MODE_RECORD) {
                // see onRecordDataDidRead seek nLatencyFix+nLatencyPlay
                // 提前开始录音，seek时由于调整延迟会丢弃一些有用数据
                nLatencyAdj =
                        mAudioTrackPlayer.getDeviceMinbufferMS() + mAudioTrackPlayer.getMiniBufferMS() + LATENCY_ADJ
                                + PLAY_FIX_LATENCY;
                nLatencyAdj = -nLatencyAdj;
                if (nLatencyAdj == 0) {
                    nLatencyAdj = -1000;
                }

                msec += nLatencyAdj;
                msecOffset -= nLatencyAdj;
                if (msec < 0) {
                    msec = 0;
                }
            }

            int nPlayPos = msec + msecOffset;
            if (nPlayPos < 0) {
                nPlayPos = 0;
            }

            logHelper.infoShow("learnplayer seekTo calc msec:" + msec + "msecOffset:" + msecOffset + "nLatencyAdj:"
                    + nLatencyAdj);
            mAudioTrackPlayer.seekTo(nPlayPos);

        }

        native_seekTo(msec, msecOffset);
        logHelper.infoShow("learnplayer seekTo end");

    }

    /**
     * 获取进度. 录音模式返回录音/播放进度 毫秒； 试听模式返回播放进度 毫秒； 保存模式返回处理进度百分比0-100
     * 
     * @return 当前进度
     */
    public int getCurrentPosition() {
        if (mLearnMode == LEARNPLAYER_MODE_RECORD || mLearnMode == LEARNPLAYER_MODE_LISTEN) {
            if (mAudioTrackPlayer != null) {
                int nCur = mAudioTrackPlayer.getCurrentPosition();
                if (nCur > mDurationMillisec && mDurationMillisec > 0) {
                    nCur = mDurationMillisec;
                }
                return nCur;
            } else {
                return 0;
            }
        } else {
            return native_getCurrentPosition();
        }
    }

    /**
     * 获取时长信息. 录音模式返回伴奏时长； 试听模式返回需混音内容(录音)时长； 保存混音返回需混音内容(录音)时长; 保存人声返回录音时长
     * 
     * @return 时长单位毫秒
     */
    public int getDuration() {
        return native_getDuration();
    }

    /**
     * 获取当前状态. 录音模式返回是否在录音播放； 试听模式返回是否正在播放； 保存模式返回处理是否在处理数据
     * 
     * @return 当前状态
     */
    public boolean isPlaying() {
        if (mLearnMode == LEARNPLAYER_MODE_RECORD || mLearnMode == LEARNPLAYER_MODE_LISTEN) {
            if (mAudioTrackPlayer != null) {
                return mAudioTrackPlayer.isPlaying();
            } else {
                return false;
            }
        } else {
            return native_isPlaying();
        }
    }

    /**
     * 设置混音比例，人声占总音量的比例0-100. 录音模式人声占总音量的比例0-100； 试听模式人声占总音量的比例0-100； 保存混音模式人声占总音量的比例0-100 保存人声不处理
     * 
     * @param ratio人声占比0 -100
     */
    /*
     * public void setVoiceBackgroundMix(int ratio){
     * 
     * }
     */
    /**
     * 设置混音比例.参数100不调整，0静音，200放大 录音模式设置人声占比，人声按需求<100低于伴奏，伴奏设置100不调整； 试听模式设置人声背景声比例； 保存混音模式同试听模式 保存人声不处理
     * 
     * @param voice人声自身调整0 -200
     * @param background伴奏自身占比0 -200
     */

    public void setVoiceBackgroundMix(int voice, int background) {
        native_setVoiceBackgroundMix(voice, background);
    }

    /**
     * 录音模式播放类型.
     * 
     * @see com.baidu.util.learncore.LearnPlayer.switchPlayBackgroundType
     */
    public static final int LEARNPLAYER_PLAYBACKGROUND_ORIGINAL = 0; // 原唱

    public static final int LEARNPLAYER_PLAYBACKGROUND_BACKGROUND = 1; // 伴奏

    /**
     * 切换原唱伴奏. 录音模式设置回放类型是原唱还是伴奏； 试听模式无效； 保存模式无效
     * 
     * @param type type是伴奏还是原唱
     */
    public void switchPlayBackgroundType(int type) {
        native_switchPlayBackgroundType(type);
    }

    /**
     * 获取评分. 录音模式获取指定时间段演唱得分； 试听模式无效； 保存模式无效
     * 
     * @param msecPos msecPos块评分起点
     * @param msecLen msecLen块评分长度
     * @return 用户该块得分(0-1.0)
     */
    public float getScore(int msecPos, int msecLen) {
        return native_getScore(msecPos, msecLen);
    }

    /**
     * 获取有效的声音值. 录音模式获取指定时间段声音的有效值； 试听模式无效； 保存模式无效
     * 
     * @param msecPos msecPos块声音有效的起点
     * @param msecLen msecLen块声音有效的长度
     * @return 用户录唱某一段的声音有效的平均值(0-1.0)
     */
    public float getValids(int msecPos, int msecLen) {
        return native_getValids(msecPos, msecLen);
    }

    /**
     * 获取当前录音音量. 录音模式获取当前录音音量； 试听模式无效； 保存模式无效
     * 
     * @return 音量范围(0-1.0)
     */
    public float getRecordLevel() {
        return native_getRecordLevel();
    }

    // ==============环境混响===============
    /**
     * 获取环境混响音效数量
     * 
     * preset names 0 无 1 演唱会 2 卧室 3 石屋 4 剧院 5 洞穴 6 空巷 7 城市 8 停车场 9 客厅 10 礼堂 11 大殿
     * 
     * @return 返回环境混响音效数量
     */
    public int getReverbNumberOfPresets() {
        return native_getReverbNumberOfPresets();
    }

    /**
     * 是否启用混响
     * 
     * @param bEnable bEnable是否启用
     */
    public void enableReverbEffect(boolean bEnable) {
        native_enableReverbEffect(bEnable);
    }

    /**
     * 获取当前预设
     * 
     * @return 当前预设值, 从 0 到 getReverbNumberOfPresets()-1.
     */
    public int getReverbCurrentPreset() {
        return native_getReverbCurrentPreset();
    }

    /**
     * 设置当前混响预设.
     * 
     * @param 设置预设值从0 到 getReverbNumberOfPresets()-1, 参考 "preset names".
     */
    public void useReverbPreset(int preset) {
        native_useReverbPreset(preset);
    }

    /**
     * 停止，各模式均适用，用于停止当前工作。执行完该方法需要重新执行prepare()，不需要重新setDataSource() 每个模式结束后对该歌曲切换模式需要调用该方法
     */
    // public void stop() throws IllegalStateException {
    // if (mAudioTrackPlayer != null) {
    // mAudioTrackPlayer.reset();
    // }
    //
    // if (mLearnMode == LEARNPLAYER_MODE_RECORD) {
    // RecorderManager.sharedPreferenceRecorderManager().stopRecord();
    // }
    //
    // native_reset();
    // // make sure none of the listeners get called anymore
    // mEventHandler.removeCallbacksAndMessages(null);
    // }

    /**
     * 重置LearnPlayer。各模式均适用，执行完该方法需要重新执行prepare()，不需要重新setDataSource() 每个模式结束后对该歌曲切换模式需要调用该方法
     */
    public void reset() throws IllegalStateException {
        if (mAudioTrackPlayer != null) {
            mAudioTrackPlayer.reset();
        }

        if (mLearnMode == LEARNPLAYER_MODE_RECORD) {
            logHelper.infoShow("reset");
            RecorderManager.sharedPreferenceRecorderManager().stopRecord();
            mLatencyMS = 0;
            mCalcLatencyIndex = 0;
        }

        // make sure none of the listeners get called anymore
        mEventHandler.removeCallbacksAndMessages(null);

        native_reset();

    }

    /**
     * 释放LearnPlayer。需要重新创建一个新的learnplayer
     */

    public void release() throws IllegalStateException {
        onRecycleResource();
    }

    /**
     * 设置学歌完成保存混音文件全路径。
     * 
     * @param strPath 混音文件全路径
     */
    public void setMixFilePath(String strPath) {
        native_setMixFilePath(strPath);
    }

    /**
     * 设置保存录制人声压缩文件全路径。
     * 
     * @param strPath 人声文件全路径
     */
    public void setVoiceFilePath(String strPath) {
        native_setVoiceFilePath(strPath);
    }

    /**
     * 设置铁肺唱将和高音王子的阈值。prepare前调用，录唱模式有效
     * 
     * @param nType 类型
     * @param nThresholdValue 阈值 铁肺唱将(0-1000) 阈值 高音王子(0-20000单位Hz)
     * @param nDurationMillisc持续时间
     */
    public void setAudioEffectParam(int nType, int nThresholdValue, int nDurationMillisc) {
        native_setAudioEffectScoreParam(nType, nThresholdValue, nDurationMillisc);
    }

    private void onInit() {
        // onCreateBufferPool();
        mIsRecycle = false;
        mLatencyMS = 0;
        mCalcLatencyIndex = 0;
        // enableGlobalLog(LOGGABLE);
        onCreateHandlerThread();
    }

    private void onCreateHandlerThread() {
        Looper looper;
        if ((looper = Looper.myLooper()) != null) {
            mEventHandler = new EventHandler(this, looper);
        } else if ((looper = Looper.getMainLooper()) != null) {
            mEventHandler = new EventHandler(this, looper);
        } else {
            mEventHandler = null;
        }
    }

    private void onRecycleResource() {
        if (!mIsRecycle) {
            if (mAudioTrackPlayer != null) {
                mAudioTrackPlayer.release();
            }

            if (mLearnMode == LEARNPLAYER_MODE_RECORD) {
                RecorderManager.sharedPreferenceRecorderManager().destroy();
            }

            native_release();
            mIsRecycle = true;
        }
    }

    @Override
    protected void finalize() throws Throwable {
        // TODO 自动生成的方法存根
        onRecycleResource();
        super.finalize();
    }

    private int mLastPlayPosMS = 1;
    private int mLastRecordPosMS = 0;

    @Override
    public boolean onRecordDataDidRead(byte[] buffer, int nSize) {
        int nCurPlayPosMS = getCurrentPosition();
        int nCurRecordPosMS = RecorderManager.sharedPreferenceRecorderManager().getTotalRecordMs();
        // logHelper.infoShow("onRecordDataDidRead curplay:" + nCurPlayPosMS);

        if (mCalcLatencyIndex < LATENCY_COUNT && nCurPlayPosMS > 1 && nCurRecordPosMS > 0
                && mLastPlayPosMS != nCurPlayPosMS && mLastRecordPosMS != nCurRecordPosMS) {
            mLastPlayPosMS = nCurPlayPosMS;
            mLastRecordPosMS = nCurRecordPosMS;
            mLatencyMS += nCurPlayPosMS - nCurRecordPosMS;
            if (++mCalcLatencyIndex == LATENCY_COUNT) {
                mLatencyMS /= LATENCY_COUNT;
                logHelper.infoShow("mLatencyMS_rp:" + mLatencyMS);

                int nLatencyAdj = mAudioTrackPlayer.getDeviceMinbufferMS() + LATENCY_ADJ;
                mLatencyMS -= nLatencyAdj;
                int nLatencyFix = -(nLatencyAdj + PLAY_FIX_LATENCY);
                int nLatencyPlay = -mAudioTrackPlayer.getMiniBufferMS();
                logHelper.infoShow("mLatencyMS:" + mLatencyMS + "play fix:" + nLatencyFix);
                native_setRecordPlayLatency(mLatencyMS, nLatencyPlay, nLatencyFix);

                mLastPlayPosMS = 1;
                mLastRecordPosMS = 0;

            }
        }

        native_getRecordBuffer(buffer, nSize);

        return true;
    }

    @Override
    public void onRecordErrorNotify(int nRecordErrorCode) {
        if (RecordError.AUDIO_RECORD_PARAM_INVALID == nRecordErrorCode) {
            sendMessage(MEDIA_ERROR, MEDIA_ERROR_RECORD_FMT, 0, null);
        } else if (RecordError.AUDIO_RECORD_PARAM_UNSUPPORTED == nRecordErrorCode) {
            sendMessage(MEDIA_ERROR, MEDIA_ERROR_RECORD_HW, 0, null);
        } else if (RecordError.AUDIO_RECORD_UNINIT == nRecordErrorCode) {
            sendMessage(MEDIA_ERROR, MEDIA_ERROR_RECORD_UNINIT, 0, null);
        } else if (RecordError.AUDIO_RECORD_NODATA == nRecordErrorCode) {
            sendMessage(MEDIA_ERROR, MEDIA_ERROR_RECORD_UNINIT, 0, null);
        }
    }

    public int getRenderBuffer(byte[] buffer, int bsize) {
        return native_getRenderBuffer(buffer, bsize);
    }

    private static void postEventFromNative(Object learnplayer_ref, int what, int arg1, int arg2, Object obj) {
        logHelper.infoShow("postEventFromNative");
        LearnPlayer ap = (LearnPlayer) learnplayer_ref;
        logHelper.debugShow("postEventFromNative what:" + what);
        if (ap != null && ap.mEventHandler != null) {
            logHelper.infoShow("obtainMessage start");
            Message m = ap.mEventHandler.obtainMessage(what, arg1, arg2, obj);
            logHelper.infoShow("obtainMessage end");
            ap.mEventHandler.sendMessage(m);
        }
    }

    /*
     * Do not change these values without updating their counterparts in include/media/mediaplayer.h!
     */
    private static final int MEDIA_NOP = 0; // interface test message
    private static final int MEDIA_PREPARED = 1;
    private static final int MEDIA_PLAYBACK_COMPLETE = 2;
    private static final int MEDIA_BUFFERING_UPDATE = 3;
    private static final int MEDIA_SEEK_COMPLETE = 4;
    private static final int MEDIA_PROGRESS = 5;
    private static final int MEDIA_ERROR = 100;
    private static final int MEDIA_INFO = 200;

    private class EventHandler extends Handler {
        private LearnPlayer mLearnPlayer = null;

        public EventHandler(LearnPlayer ap, Looper looper) {
            super(looper);
            mLearnPlayer = ap;
        }

        @Override
        public void handleMessage(Message msg) {

            if (mLearnPlayer.mNativeContext == 0) {
                logHelper.warnShow("mediaplayer went away with unhandled events");
                return;
            }

            logHelper.debugShow("msg = " + msg.what);
            switch (msg.what) {
                case MEDIA_PREPARED:
                    return;
                case MEDIA_PLAYBACK_COMPLETE:
                    if (mAudioTrackPlayer != null) {
                        mAudioTrackPlayer.OnPlayComplete();
                    }

                    if (mOnCompletionListener != null)
                        mOnCompletionListener.onCompletion(mLearnPlayer);
                    return;
                case MEDIA_BUFFERING_UPDATE:
                    return;
                case MEDIA_SEEK_COMPLETE:
                    boolean bSuc = true;
                    if (mAudioTrackPlayer != null) {
                        if (msg.arg1 == 0)
                            bSuc = true;
                        else
                            bSuc = false;
                        mAudioTrackPlayer.OnseekComplete(bSuc);
                    }
                    if (mOnSeekCompleteListener != null)
                        mOnSeekCompleteListener.onSeekComplete(mLearnPlayer, bSuc);
                    return;
                case MEDIA_PROGRESS:
                    if (mOnSavingUpdateListener != null)
                        mOnSavingUpdateListener.onSavingUpdate(mLearnPlayer, msg.arg1); // arg1 percent
                    return;
                case MEDIA_ERROR:
                    // For PV specific error values (msg.arg2) look in
                    // opencore/pvmi/pvmf/include/pvmf_return_codes.h
                    logHelper.errorShow("Error (" + msg.arg1 + "," + msg.arg2 + ")");
                    boolean error_was_handled = false;
                    if (mOnErrorListener != null) {
                        error_was_handled = mOnErrorListener.onError(mLearnPlayer, msg.arg1, msg.arg2);
                    }
                    if (mOnCompletionListener != null && !error_was_handled) {
                        mOnCompletionListener.onCompletion(mLearnPlayer);
                    }
                    return;

                case MEDIA_INFO:
                    // For PV specific code values (msg.arg2) look in
                    // opencore/pvmi/pvmf/include/pvmf_return_codes.h
                    logHelper.infoShow("Info (" + msg.arg1 + "," + msg.arg2 + ")");
                    if (msg.arg1 == INFO_GOOD_SINGER || msg.arg1 == INFO_HIGH_PITCH) {
                        logHelper.infoShow("mOnHowToSingListener");
                        int nType = (msg.arg1 == INFO_GOOD_SINGER ? INFO_GOOD_SINGER : INFO_HIGH_PITCH);
                        if (mOnHowToSingListener != null) {
                            mOnHowToSingListener.onGoodVoiceSing(nType);
                        }
                        return;
                    }
                    if (mOnInfoListener != null) {
                        mOnInfoListener.onInfo(mLearnPlayer, msg.arg1, msg.arg2);
                        return;
                    }
                    // No real default action so far.
                    break;

                case MEDIA_NOP: // interface test message - ignore
                    break;

                default:
                    logHelper.errorShow("Unknown message type " + msg.what);
                    return;
            }
        }
    }

    public void sendMessage(int what, int arg1, int arg2, Object obj) {
        Message m = mEventHandler.obtainMessage(what, arg1, arg2, obj);
        mEventHandler.sendMessage(m);
    }

    // native method
    private static native final void native_init();

    private native final void native_setup(Object weak_this);

    private native final void native_release();

    private native final void native_reset();

    // 设置音频处理临时路径
    private static native boolean native_setTempPath(String strPath);

    private native void native_setVoiceFilePath(String strPath);

    // 设置歌词路径
    private native boolean native_setLyricPath(String strPath);

    /**
     * Sets the data source (FileDescriptor) to use. It is the caller's responsibility to close the file descriptor. It
     * is safe to do so as soon as this call returns.
     * 
     * @param fd the FileDescriptor for the file you want to play
     * @param offset the offset into the file where the data to be played starts, in bytes
     * @param length the length in bytes of the data to be played
     * @throws IllegalStateException if it is called in an invalid state
     */
    private native void native_setDataSource(FileDescriptor fd, long offset, long length) throws IOException,
            IllegalArgumentException, IllegalStateException;

    /**
     * Sets the data source (file-path or http/rtsp URL) to use.
     * 
     * @param path the path of the file, or the http/rtsp URL of the stream you want to play
     * @throws IllegalStateException if it is called in an invalid state
     */
    private native void native_setDataSource(String path) throws IOException, IllegalArgumentException,
            IllegalStateException;

    private native void native_prepare(int mode) throws IOException, IllegalStateException;

    private native void native_getRecordBuffer(byte[] buffer, int bsize);

    private native int native_getDuration();

    private native int native_getCurrentPosition();

    private native void native_switchPlayBackgroundType(int type);

    private native float native_getScore(int msecPos, int msecLen);

    private native float native_getValids(int msecPos, int msecLen);

    private native float native_getRecordLevel();

    private native void native_setVoiceBackgroundMix(int voice, int background);

    private native void native_setMixFilePath(String strPath);

    private native void native_seekTo(int msec, int msecOffset) throws IllegalStateException;

    private native void native_pause() throws IllegalStateException;

    private native void native_start() throws IllegalStateException;

    private native boolean native_isPlaying();

    public static native void native_enableGlobalLog(boolean bEnableLog);

    private native int native_getRenderBuffer(byte[] buffer, int bsize);

    private native void native_setPlayerFlags(int nFlags);

    private native int native_getReverbNumberOfPresets();

    private native void native_enableReverbEffect(boolean bEnable);

    private native int native_getReverbCurrentPreset();

    private native void native_useReverbPreset(int preset);

    private native int native_getRenderSampleRate();

    private native int native_getRenderChannels();

    private native int native_getRenderBitsPerSample();

    private native void native_keepRecordData(boolean keep);

    // the latency between record and playback, the minus of record and playback
    private native void native_setRecordPlayLatency(int latencyMS, int latencyPlayMS, int latencyPlayFixMS);

    private native void native_setAudioEffectScoreParam(int nType, int nThresholdValue, int nDurationMillisc);

    private native void native_cancelEchoProcess(boolean bCancel);
}