package com.baidu.util.localmatch;

import java.io.File;
import java.io.IOException;
import java.lang.ref.WeakReference;
import java.util.UUID;

import net.sf.json.JSONObject;
import android.content.Context;
import android.content.IntentFilter;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.location.LocationListener;
import android.media.AudioFormat;
import android.net.ConnectivityManager;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.telephony.TelephonyManager;
import android.util.Log;

import com.baidu.utility.LogHelper;
import com.baidu.utility.MD5Utils;
import com.baidu.utility.StringUtils;
import com.baidu.utility.api.ApiConfig;
import com.baidu.utility.api.ApiController;
import com.baidu.utility.api.ApiParser;
import com.baidu.util.localmatch.*;

public class LocalMatch {

    private final static String TAG = "LocalMatch";
    private static final LogHelper logHelper = new LogHelper(TAG);
    
    private int mNativeContext; // accessed by native methods

  
    LocalMatchListener mLocalMatchListener = null;
    private EventHandler mEventHandler;
    private static Context longLifeContext = null;
    
    private ApiController mApiController;
    private ApiParser mApiParser;
    
    private String mSysType;
    private String mVersion;
    private String mChannel;
    
    static {
        System.loadLibrary("coreffmpeg");
        System.loadLibrary("ijksoundtouch");
        System.loadLibrary("basecore");
        System.loadLibrary("localmatch");
        native_init();
    }

    public LocalMatch(Looper eventlooper) {
        if (eventlooper != null) {
            logHelper.infoShow("VoiceRecognition eventlooper out");
            mEventHandler = new EventHandler(this, eventlooper);
        } else {
            logHelper.infoShow("AudioPlayer eventlooper null");
            Looper looper;
            if ((looper = Looper.myLooper()) != null) {
                mEventHandler = new EventHandler(this, looper);
            } else if ((looper = Looper.getMainLooper()) != null) {
                mEventHandler = new EventHandler(this, looper);
            } else {
                mEventHandler = null;
            }
        }

        /*
         * Native setup requires a weak reference to our object. It's easier to
         * create it here than in C++.
         */
        native_setup(new WeakReference<LocalMatch>(this));

        setEnvironmentInfo();
        
		mApiController = new ApiController();
		if(mApiController == null){
			return;
		}
		mApiController.setOnApiControllerListener(mOnApiControllerListener);
		
		mApiParser = new ApiParser();
		if(mApiParser == null){
			return;
		}
      
    }

    public LocalMatch() {
        this(null);
    }
    
//    public static void enableGlobalLog(boolean loggable) {
//        logHelper.enableLog(loggable);
//        int flag = ((loggable) ? 1 : 0);
//        native_enableGlobalLog(flag);
//    }

    /*
     * set context, call this method before the instance of Audioplayer is
     * created
     */
    public static void setLongLifeContext(Context context) {
        longLifeContext = context;
    }


    /**
     * set local match listener
     * @param listener
     */
    public void setLocalMatchListener(LocalMatchListener listener) {
    	//logHelper.infoShow("localmatch_Listener_setLocalMatchListener ");
        this.mLocalMatchListener = listener;
    }

    //初始化本地匹配资源
    /*
     *  sys : "android"
     *  version: android 的版本号 "6.0.7.0"
     *  channel: 渠道
     *   
     */
    public boolean initLocalMatch(String sys, String version, String channel) {
    	if(!StringUtils.isEmpty(sys)){
    		mSysType = sys;
    	}else{
    		mSysType = "android";
    	}
    	
    	if(!StringUtils.isEmpty(version)){
    		mVersion = version;
    	}
    	
    	if(!StringUtils.isEmpty(channel)){
    		mChannel = channel;
    	}
    	
    	if(StringUtils.isEmpty(mSysType) || StringUtils.isEmpty(mSysType) || StringUtils.isEmpty(channel)){
    		Log.e(TAG, "local_match_check param= ");
    		return false;
    	}
    	
    	mApiController.parserInit(mSysType, mVersion, mChannel);
    	native_match_init();
    	return true;
    }
    
    // 开始本地本地匹配，需要设置setLocalMatchListener代理方法，并传入实现对像监听事件
    // 返回值参考 onLocalMatchClientResult
    public boolean startLocalMatch(String filePath)  throws IllegalStateException, IOException{
    	
    	Log.i(TAG, "local_match_filePath= "+filePath);
    	String songid = null;
    	String result = null;
    	if(StringUtils.isEmpty(filePath)){
    		Log.e(TAG, "onError filePath = null ");
    		return false;
    	}
    		
    	File file = new File(filePath);  
    	if (!file.exists()) {  
    		Log.e(TAG, "onError file not exists ");
    		return false; 
    	} 
    	
    	String fileMd5 = MD5Utils.getFileMD5(file);
    	String response = null;
        response = mApiController.getSongIdByMd5(fileMd5);
    	if(!StringUtils.isEmpty(response)){
    		songid = mApiParser.getSongIdFromResponse(response);
    		Log.e(TAG, "local_match_md5_songid="+songid);
    	}
 
    	if(!StringUtils.isEmpty(songid)){
    		setLocalMatchToResult(LocalMatchListener.LocalMatchFromMd5,filePath,songid);
    	}

    	//md5 not find ,find by keyword
    	if(StringUtils.isEmpty(songid)){
    		String info[] = native_get_tag_from_file(filePath);
        	
        	String singer = info[0];
        	String songname = info[1];
        	
        	int durationSec = 0;
        	int durationMicrosecond = StringUtils.toInt(info[2],0);
        	if(durationMicrosecond != 0){
        		durationSec = durationMicrosecond / 1000000;
        	}
        	String duration =StringUtils.intToString(durationSec); // us -> s 
        	Log.e(TAG, "local_match_native_get_tag_from_file singer="+singer+" songname="+songname+" duration="+duration+"s");
        	
        	if(!StringUtils.isEmpty(singer) && !StringUtils.isEmpty(songname) && !StringUtils.isEmpty(duration)){
        		response = (String) mApiController.getSongIdByKeyWord(singer,songname,duration);
        		if(!StringUtils.isEmpty(response)){
        			songid = mApiParser.getSongIdFromResponse(response);
        			Log.e(TAG, "local_match_keyword_songid="+songid);
        		}
        	}
        	
        	if(!StringUtils.isEmpty(songid)){
        		setLocalMatchToResult(LocalMatchListener.LocalMatchFromKeyWord,filePath,songid);
        	}
    	}
    	
    	//md5 ,keyword not find ,find by audiofinger
    	if(StringUtils.isEmpty(songid)){
    		Log.e(TAG, "local_match_audiofp_filePath="+filePath);
    		native_match_file(filePath);
    	}
    	
    	return true;
   }
    
	public String setLocalMatchToResult(int from ,String path, String songid){
		String result = null;
		JSONObject jsonObject = new JSONObject();
		jsonObject.put("file_path", path);
		jsonObject.put("song_id", songid); 
		result = jsonObject.toString();
		mLocalMatchListener.onLocalMatchClientResult(from, LocalMatchListener.ELocalMatchClientErrorStatusSuccess, result);
		return result; 
	}
 
 // 结束本地匹配
    public void releaseLocalMatch()  throws IllegalStateException{
    	 native_match_release();
    	 longLifeContext = null;
    }
   
    /*
     * Do not change these values without updating their counterparts in
     * include/media/mediaplayer.h!
     */
    private static final int ELocalMatchClientErrorStatus = 200;
    

    private class EventHandler extends Handler {
        private LocalMatch mAudioPlayer;

        public EventHandler(LocalMatch ap, Looper looper) {
            super(looper);
            mAudioPlayer = ap;
        }

        @Override
        public void handleMessage(Message msg) {
           // logHelper.infoShow("localmatch_handleMessage msg = " + msg.what);

            if (mAudioPlayer.mNativeContext == 0) {
                logHelper
                        .warnShow("localmatch_error_mediaplayer went away with unhandled events");
                return;
            }

          //  logHelper.infoShow("localmatch_Listener_msg = " + msg.what );
            switch (msg.what) {
            case ELocalMatchClientErrorStatus:
            {
                String resultString = (String) msg.obj;
                int errorStatus = msg.arg1;
                if (mLocalMatchListener != null){
                    logHelper.infoShow("localmatch_Listener_msg msg.arg1" + msg.arg1 + "what= " + msg.what + " msg.obj"+msg.obj);
                    mLocalMatchListener.onLocalMatchClientResult(LocalMatchListener.LocalMatchFromAudioFingerPrinting, errorStatus, resultString);
                }else{
                   logHelper.infoShow("localmatch_Listener_msg = null");
                }
                

            }
                 break;
            default:
                logHelper.errorShow("Unknown message type " + msg.what);
                break;
            }

            logHelper.infoShow("msg = " + msg.what + " end");
        }
    }

    private void SendMessage(int what, int arg1, int arg2, Object obj) {
        if (mEventHandler != null) {
            Message m = mEventHandler.obtainMessage();
            m.what = what;
            m.arg1 = arg1;
            m.arg2 = arg2;
            m.obj = obj;
            mEventHandler.sendMessage(m);
        }
    }

    private static void setEnvironmentInfo() {
        if (longLifeContext != null) {
        	if (hasReadPhoneStatePermission()) {
                final TelephonyManager tm = (TelephonyManager) longLifeContext
                        .getSystemService(Context.TELEPHONY_SERVICE);
                final String tmDevice, tmSerial, androidId;
                tmDevice = "" + tm.getDeviceId();
                tmSerial = "" + tm.getSimSerialNumber();
                androidId = ""
                        + android.provider.Settings.Secure.getString(
                                longLifeContext.getContentResolver(),
                                android.provider.Settings.Secure.ANDROID_ID);
                UUID deviceUuid = new UUID((androidId + tmDevice).hashCode(),
                        ((long) (tmDevice + tmSerial).hashCode() << 32)
                                | (tmSerial + androidId).hashCode());
                String ci = deviceUuid.toString();
                String machine = android.os.Build.MODEL;
                String sys = "Android";
                String sysver = android.os.Build.VERSION.RELEASE;
                String appname = longLifeContext.getPackageName();
                String appver = "";
                try {
                    PackageManager packageManager = longLifeContext
                            .getPackageManager();
                    PackageInfo packageInfo = packageManager.getPackageInfo(
                            longLifeContext.getPackageName(), 0);
                    appver = packageInfo.versionName;
                    if (appver == null) {
                        appver = "";
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
                logHelper.debugShow("SetEnvironmentInfo:" + ci + "," + machine + ""
                        + "," + sys + "," + sysver + "," + appname + "," + appver
                        + ",");
                native_setEnvironmentInfo(ci, machine, sys, sysver, appname, appver);
        	} else {
        		logHelper.debugShow("do not have read state permission");
        		native_setEnvironmentInfo("", "", "Android", "", "", "");
        	}
        }
    }
    
    private static final String READ_PHONE_STATE_PERMISSION = "android.permission.READ_PHONE_STATE";
    private static boolean hasReadPhoneStatePermission() {
        int perm = longLifeContext.checkCallingOrSelfPermission(READ_PHONE_STATE_PERMISSION);
        return perm == PackageManager.PERMISSION_GRANTED;
    }

    /**
     * Called from native code when an interesting event happens. This method
     * just uses the EventHandler system to post the event back to the main app
     * thread. We use a weak reference to the original AudioPlayer object so
     * that the native code is safe from the object disappearing from underneath
     * it. (This is the cookie passed to native_setup().)
     */
    private static void postEventFromNative(Object mediaplayer_ref, int what,
            int arg1, int arg2, Object obj) {

       // logHelper.infoShow("localmatch_postEventFromNative what"+what);
        LocalMatch ap = (LocalMatch) ((WeakReference<?>) mediaplayer_ref)
                .get();
        if (ap == null) {
            logHelper.infoShow("postEventFromNative ap null");
            return;
        }

        if (ap.mEventHandler == null) {
        	  logHelper.infoShow("localmatch_postEventFromNative_mEventHandler = null");
        }
        
        if (ap.mEventHandler != null) {
          //  logHelper.infoShow("localmatch_postEventFromNative_message_has_send");
            Message m = ap.mEventHandler.obtainMessage(what, arg1, arg2, obj);
            ap.mEventHandler.sendMessage(m);
        }
    }

    private static native final void native_init();
    
    private static native final void native_enableGlobalLog(int flag);
    
    private native final void native_setup(Object mediaplayer_this);
    
    /*
    private static native void native_setProxy(String host, int port, String user,
            String passwd);

    private static native void native_setUserAgent(String ua)
            throws IllegalArgumentException;

    private static native void native_setPartner(String partner)
            throws IllegalArgumentException;

    private static native void native_setCustomHttpHeader(
            String keys, String values, String seperator) throws IllegalArgumentException;
    */
    private native void native_match_init();
    private native void native_match_file(String path);
    private native  String [] native_get_tag_from_file(String path);
    private native void native_match_release();

    private static native void native_setEnvironmentInfo(String ci, String machine,
            String sys, String sysver, String appname, String appver);
   

	private ApiController.OnApiControllerListener mOnApiControllerListener = new ApiController.OnApiControllerListener() {

		@Override
		public void onSuccess(int status, String what, int id) {
			Log.e(TAG, "onSuccess id " + status + " id " + id);
			switch (id) {
				case ApiConfig.API_LOCALMATCH_MD5_ID: {
				
					break;
				}
				case ApiConfig.API_LOCALMATCH_KEYWORD_ID: {
				
					break;
				}
				default:
					break;
			}
		}

		public void onError(int status, String what, int id) {
			Log.i(TAG, "onError status " + status + " what= " + what + " id= " + id);
			switch (id) {
				case ApiConfig.API_LOCALMATCH_MD5_ID: {
					Log.i(TAG, "onError API_LOCALMATCH_MD5_ID ");
					break;
				}
				case ApiConfig.API_LOCALMATCH_KEYWORD_ID: {
					Log.i(TAG, "onError API_LOCALMATCH_KEYWORD_ID");
					break;
				}
				default:
					break;
			}

		}

		public void OnProgress(float progress, long total, int id) {

		}
	};


}
