package com.baidu.util.voicerecognition;

import java.lang.ref.WeakReference;
import java.util.UUID;

import android.content.Context;
import android.content.IntentFilter;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.media.AudioFormat;
import android.net.ConnectivityManager;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.telephony.TelephonyManager;

import com.baidu.musicRecorder.RecordError;
import com.baidu.musicRecorder.RecorderManager;
import com.baidu.utility.LogHelper;
import com.utility.publicInterface.RecordListener;


public class VoiceRecognition implements RecordListener {

    private final static String tag = "VoiceRecognition";
    private static final LogHelper logHelper = new LogHelper(tag);
    
    private int mNativeContext; // accessed by native methods

    public static final int VOICE_RECOGNITION_SAMPLERATE = 44100;
    public static final int VOICE_RECOGNITION_CHANNEL = 2;
    public static final int VOICE_RECOGNITION_BITSPERSAMPLE = 16;

    VoiceRecognitionListener mRecognitionListener = null;
    private EventHandler mEventHandler;
    private static Context longLifeContext = null;
    
    static {
        System.loadLibrary("basecore");
        System.loadLibrary("voicerecognition");
        native_init();
    }

    public VoiceRecognition(Looper eventlooper) {
        if (eventlooper != null) {
            logHelper.infoShow("VoiceRecognition eventlooper out");
            mEventHandler = new EventHandler(this, eventlooper);
        } else {
            logHelper.infoShow("AudioPlayer eventlooper null");
            Looper looper;
            if ((looper = Looper.myLooper()) != null) {
                mEventHandler = new EventHandler(this, looper);
            } else if ((looper = Looper.getMainLooper()) != null) {
                mEventHandler = new EventHandler(this, looper);
            } else {
                mEventHandler = null;
            }
        }

        /*
         * Native setup requires a weak reference to our object. It's easier to
         * create it here than in C++.
         */
        native_setup(new WeakReference<VoiceRecognition>(this), 
                VOICE_RECOGNITION_SAMPLERATE, VOICE_RECOGNITION_CHANNEL, VOICE_RECOGNITION_BITSPERSAMPLE);

        setEnvironmentInfo();
        
        RecorderManager.sharedPreferenceRecorderManager().setRecordListener(this);
        RecorderManager.sharedPreferenceRecorderManager().setRecordSettings(
                VOICE_RECOGNITION_SAMPLERATE, 
                (VOICE_RECOGNITION_CHANNEL == 1) ? AudioFormat.CHANNEL_IN_MONO : AudioFormat.CHANNEL_IN_STEREO, 
                (VOICE_RECOGNITION_BITSPERSAMPLE == 16) ? AudioFormat.ENCODING_PCM_16BIT : AudioFormat.ENCODING_PCM_8BIT);
        RecorderManager.sharedPreferenceRecorderManager().setRecordBufferSize(
                VOICE_RECOGNITION_SAMPLERATE * VOICE_RECOGNITION_CHANNEL * VOICE_RECOGNITION_BITSPERSAMPLE / 8 / 20);
    }

    public VoiceRecognition() {
        this(null);
    }
    
    public static void enableGlobalLog(boolean loggable) {
        logHelper.enableLog(loggable);
        int flag = ((loggable) ? 1 : 0);
        native_enableGlobalLog(flag);
    }

    /*
     * set context, call this method before the instance of Audioplayer is
     * created
     */
    public static void setLongLifeContext(Context context) {
        longLifeContext = context;
    }


    public void setRecognitionListener(VoiceRecognitionListener listener) {
        this.mRecognitionListener = listener;
    }

    // 开始语音识别，需要实现VoiceRecognitionClientDelegate代理方法，并传入实现对像监听事件
    // 返回值参考 TVoiceRecognitionStartWorkResult
    public void startVoiceRecognition(int resultCountNeeded)  throws IllegalStateException{
        logHelper.debugShow("startVoiceRecognition:" + resultCountNeeded);
        native_start(resultCountNeeded);
        
        if (!RecorderManager.sharedPreferenceRecorderManager()
                .prepareRecord()) {
            throw new IllegalStateException("recorder not prepared !");
        }
        RecorderManager.sharedPreferenceRecorderManager().create();
        RecorderManager.sharedPreferenceRecorderManager().startRecord();
   }
 
 // 结束本次语音识别
    public void stopVoiceRecognition() throws IllegalStateException {
        logHelper.debugShow("stopVoiceRecognition");
        native_stop();
        RecorderManager.sharedPreferenceRecorderManager().stopRecord();
    }
    // 获取可能的结果
    public String getResultMaybe(int count)  throws IllegalStateException {
        return native_get_result_maybe(count);
    }
    
    // 获取当前录音音量大小，min -48.0dB, max 0.0dB
    public float getCurrentRecorddB()  throws IllegalStateException {
        return native_get_current_record_dB();
    }
    
    public void releaseVoiceRecognition() {
        logHelper.debugShow("releaseVoiceRecognition");
        RecorderManager.sharedPreferenceRecorderManager().destroy();
        native_release();
        longLifeContext = null;
    }

    // set proxy, if not need strUser and strPwd, you can set them nil
    public static void setProxy(String host, int port, String user,
            String passwd) {
        native_setProxy(host, port, user, passwd);
    }

    public static void setUserAgent(String ua) throws IllegalArgumentException {
        logHelper.debugShow("setUserAgent:" + ua);
        native_setUserAgent(ua);
    }

    // set uniqueIdentifier of partner, music SDK or lebo SDK should call this
    // method, should be called after initP2pEnvironment is called
    public static void setPartner(String partner)
            throws IllegalArgumentException {
        logHelper.debugShow("setPartner:" + partner);
        native_setPartner(partner);
    }

    /**
     * set user custom http header, now used by China Mobile 
     * @param keys etc "ClientID" or "ClientID@@@@ClientVer"
     * @param values etc "MyClient31-User67328" or"MyClient31-User67328@@@@1.0"
     * @param seperator etc "" or "@@@@"
     * @throws IllegalArgumentException
     */
    public static void setCustomHttpHeader(String keys, String values, String seperator)
            throws IllegalArgumentException {
        logHelper.debugShow("setCustomHttpHeader keys:" + keys 
                + " values: " + values + " seperator: " + seperator);
        native_setCustomHttpHeader(keys, values, seperator);
    }
    /*
     * Do not change these values without updating their counterparts in
     * include/media/mediaplayer.h!
     */
    private static final int EVoiceRecognitionClientErrorStatus = 200;
    

    private class EventHandler extends Handler {
        private VoiceRecognition mAudioPlayer;

        public EventHandler(VoiceRecognition ap, Looper looper) {
            super(looper);
            mAudioPlayer = ap;
        }

        @Override
        public void handleMessage(Message msg) {
            logHelper.infoShow("handleMessage msg = " + msg.what);

            if (mAudioPlayer.mNativeContext == 0) {
                logHelper
                        .warnShow("mediaplayer went away with unhandled events");
                return;
            }

            logHelper.infoShow("msg = " + msg.what + " begin");
            switch (msg.what) {
            case EVoiceRecognitionClientErrorStatus:
            {
                String resultString = (String) msg.obj;
                int errorStatus = msg.arg1;
                if (mRecognitionListener != null)
                    mRecognitionListener.onVoiceRecognitionClientResult(errorStatus, resultString);

            }
                 break;
            default:
                logHelper.errorShow("Unknown message type " + msg.what);
                break;
            }

            logHelper.infoShow("msg = " + msg.what + " end");
        }
    }

    private void SendMessage(int what, int arg1, int arg2, Object obj) {
        if (mEventHandler != null) {
            Message m = mEventHandler.obtainMessage();
            m.what = what;
            m.arg1 = arg1;
            m.arg2 = arg2;
            m.obj = obj;
            mEventHandler.sendMessage(m);
        }
    }

    private static void setEnvironmentInfo() {
        if (longLifeContext != null) {
        	if (hasReadPhoneStatePermission()) {
                final TelephonyManager tm = (TelephonyManager) longLifeContext
                        .getSystemService(Context.TELEPHONY_SERVICE);
                final String tmDevice, tmSerial, androidId;
                tmDevice = "" + tm.getDeviceId();
                tmSerial = "" + tm.getSimSerialNumber();
                androidId = ""
                        + android.provider.Settings.Secure.getString(
                                longLifeContext.getContentResolver(),
                                android.provider.Settings.Secure.ANDROID_ID);
                UUID deviceUuid = new UUID((androidId + tmDevice).hashCode(),
                        ((long) (tmDevice + tmSerial).hashCode() << 32)
                                | (tmSerial + androidId).hashCode());
                String ci = deviceUuid.toString();
                String machine = android.os.Build.MODEL;
                String sys = "Android";
                String sysver = android.os.Build.VERSION.RELEASE;
                String appname = longLifeContext.getPackageName();
                String appver = "";
                try {
                    PackageManager packageManager = longLifeContext
                            .getPackageManager();
                    PackageInfo packageInfo = packageManager.getPackageInfo(
                            longLifeContext.getPackageName(), 0);
                    appver = packageInfo.versionName;
                    if (appver == null) {
                        appver = "";
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
                logHelper.debugShow("SetEnvironmentInfo:" + ci + "," + machine + ""
                        + "," + sys + "," + sysver + "," + appname + "," + appver
                        + ",");
                native_setEnvironmentInfo(ci, machine, sys, sysver, appname, appver);
        	} else {
        		logHelper.debugShow("do not have read state permission");
        		native_setEnvironmentInfo("", "", "Android", "", "", "");
        	}
        }
    }
    
    private static final String READ_PHONE_STATE_PERMISSION = "android.permission.READ_PHONE_STATE";
    private static boolean hasReadPhoneStatePermission() {
        int perm = longLifeContext.checkCallingOrSelfPermission(READ_PHONE_STATE_PERMISSION);
        return perm == PackageManager.PERMISSION_GRANTED;
    }

    /**
     * Called from native code when an interesting event happens. This method
     * just uses the EventHandler system to post the event back to the main app
     * thread. We use a weak reference to the original AudioPlayer object so
     * that the native code is safe from the object disappearing from underneath
     * it. (This is the cookie passed to native_setup().)
     */
    private static void postEventFromNative(Object mediaplayer_ref, int what,
            int arg1, int arg2, Object obj) {

        logHelper.infoShow("postEventFromNative");
        VoiceRecognition ap = (VoiceRecognition) ((WeakReference<?>) mediaplayer_ref)
                .get();
        if (ap == null) {
            logHelper.infoShow("postEventFromNative ap null");
            return;
        }

        if (ap.mEventHandler != null) {
            logHelper.infoShow("postEventFromNative message");
            Message m = ap.mEventHandler.obtainMessage(what, arg1, arg2, obj);
            ap.mEventHandler.sendMessage(m);
        }
    }

    private static native final void native_init();
    
    private static native final void native_enableGlobalLog(int flag);
    
    private native final void native_setup(Object mediaplayer_this, int sample_rate, int channel, int bits_per_sample);
    
    private static native void native_setProxy(String host, int port, String user,
            String passwd);

    private static native void native_setUserAgent(String ua)
            throws IllegalArgumentException;

    private static native void native_setPartner(String partner)
            throws IllegalArgumentException;

    private static native void native_setCustomHttpHeader(
            String keys, String values, String seperator) throws IllegalArgumentException;
    
    private native void native_start(int resultCountNeeded)  throws IllegalStateException;
    
    private native void native_stop()  throws IllegalStateException;
    
    private native void native_release();

    private native void native_setRecordBuffer(byte[] buffer, int nSize);

    private native String native_get_result_maybe(int count)  throws IllegalStateException;
    
    private native float native_get_current_record_dB()  throws IllegalStateException;
    
    private static native void native_setEnvironmentInfo(String ci, String machine,
            String sys, String sysver, String appname, String appver);
    @Override
    public boolean onRecordDataDidRead(byte[] buffer, int nSize) {
         native_setRecordBuffer(buffer, nSize);
        return true;
    }

    @Override
    public void onRecordErrorNotify(int nRecordErrorCode) {
        if (RecordError.AUDIO_RECORD_PARAM_INVALID == nRecordErrorCode) {
            SendMessage(EVoiceRecognitionClientErrorStatus, VoiceRecognitionListener.EVoiceRecognitionClientErrorRecordFormat, 0, null);
        } else if (RecordError.AUDIO_RECORD_PARAM_UNSUPPORTED == nRecordErrorCode) {
            SendMessage(EVoiceRecognitionClientErrorStatus, VoiceRecognitionListener.EVoiceRecognitionClientErrorRecordHW, 0, null);
        } else if (RecordError.AUDIO_RECORD_UNINIT == nRecordErrorCode) {
            SendMessage(EVoiceRecognitionClientErrorStatus, VoiceRecognitionListener.EVoiceRecognitionClientErrorRecordUninit, 0, null);
        } else if (RecordError.AUDIO_RECORD_NODATA == nRecordErrorCode) {
            SendMessage(EVoiceRecognitionClientErrorStatus, VoiceRecognitionListener.EVoiceRecognitionClientErrorRecordUninit, 0, null);
        }
    }

}
